<?php declare(strict_types=1);

namespace Shopware\Core\Framework\Test\DataAbstractionLayer\Dbal;

use PHPUnit\Framework\TestCase;
use Shopware\Core\Content\Product\ProductDefinition;
use Shopware\Core\Defaults;
use Shopware\Core\Framework\Api\Context\SystemSource;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\Dbal\EntityHydrator;
use Shopware\Core\Framework\DataAbstractionLayer\Dbal\EntityReader;
use Shopware\Core\Framework\DataAbstractionLayer\DefinitionInstanceRegistry;
use Shopware\Core\Framework\DataAbstractionLayer\EntityCollection;
use Shopware\Core\Framework\DataAbstractionLayer\EntityDefinition;
use Shopware\Core\Framework\DataAbstractionLayer\Field\FkField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\Flag\ApiAware;
use Shopware\Core\Framework\DataAbstractionLayer\Field\Flag\Extension;
use Shopware\Core\Framework\DataAbstractionLayer\Field\Flag\PrimaryKey;
use Shopware\Core\Framework\DataAbstractionLayer\Field\IdField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\StringField;
use Shopware\Core\Framework\DataAbstractionLayer\FieldCollection;
use Shopware\Core\Framework\Struct\ArrayEntity;
use Shopware\Core\Framework\Struct\ArrayStruct;
use Shopware\Core\Framework\Test\DataAbstractionLayer\Field\DataAbstractionLayerFieldTestBehaviour;
use Shopware\Core\Framework\Test\DataAbstractionLayer\Field\TestDefinition\CustomFieldPlainTestDefinition;
use Shopware\Core\Framework\Test\DataAbstractionLayer\Field\TestDefinition\CustomFieldTestDefinition;
use Shopware\Core\Framework\Test\DataAbstractionLayer\Field\TestDefinition\CustomFieldTestTranslationDefinition;
use Shopware\Core\Framework\Test\DataAbstractionLayer\Field\TestDefinition\SingleEntityDependencyTestDependencyDefinition;
use Shopware\Core\Framework\Test\DataAbstractionLayer\Field\TestDefinition\SingleEntityDependencyTestDependencySubDefinition;
use Shopware\Core\Framework\Test\DataAbstractionLayer\Field\TestDefinition\SingleEntityDependencyTestRootDefinition;
use Shopware\Core\Framework\Test\DataAbstractionLayer\Field\TestDefinition\SingleEntityDependencyTestSubDefinition;
use Shopware\Core\Framework\Test\TestCaseBase\BasicTestDataBehaviour;
use Shopware\Core\Framework\Test\TestCaseBase\DatabaseTransactionBehaviour;
use Shopware\Core\Framework\Test\TestCaseBase\KernelTestBehaviour;
use Shopware\Core\Framework\Uuid\Uuid;

class EntityHydratorTest extends TestCase
{
    use KernelTestBehaviour;
    use BasicTestDataBehaviour;
    use DatabaseTransactionBehaviour;
    use DataAbstractionLayerFieldTestBehaviour;

    public function testFkExtensionFieldHydration(): void
    {
        $definition = new FkExtensionFieldTest();
        $definition->compile($this->getContainer()->get(DefinitionInstanceRegistry::class));

        $hydrator = new EntityHydrator($this->getContainer());

        $id = Uuid::randomBytes();
        $normal = Uuid::randomBytes();
        $extended = Uuid::randomBytes();

        $rows = [
            [
                'test.id' => $id,
                'test.name' => 'test',
                'test.normalFk' => $normal,
                'test.extendedFk' => $extended,
            ],
        ];

        $structs = $hydrator->hydrate(new EntityCollection(), $definition->getEntityClass(), $definition, $rows, 'test', Context::createDefaultContext());
        static::assertCount(1, $structs);

        /** @var ArrayEntity|null $first */
        $first = $structs->first();

        static::assertInstanceOf(ArrayEntity::class, $first);

        static::assertSame('test', $first->get('name'));

        static::assertSame(Uuid::fromBytesToHex($id), $first->get('id'));
        static::assertSame(Uuid::fromBytesToHex($normal), $first->get('normalFk'));

        static::assertTrue($first->hasExtension(EntityReader::FOREIGN_KEYS));
        /** @var ArrayStruct|null $foreignKeys */
        $foreignKeys = $first->getExtension(EntityReader::FOREIGN_KEYS);

        static::assertInstanceOf(ArrayStruct::class, $foreignKeys);

        static::assertTrue($foreignKeys->has('extendedFk'));
        static::assertSame(Uuid::fromBytesToHex($extended), $foreignKeys->get('extendedFk'));
    }

    public function testCustomFieldHydrationWithoutTranslationWithoutInheritance(): void
    {
        $definition = $this->registerDefinition(CustomFieldPlainTestDefinition::class);

        $hydrator = new EntityHydrator($this->getContainer());

        $id = Uuid::randomBytes();

        $rows = [
            [
                'test.id' => $id,
                'test.name' => 'example',
                'test.customFields' => '{"custom_test_text": "Example", "custom_test_check": null}',
            ],
        ];

        $structs = $hydrator->hydrate(new EntityCollection(), $definition->getEntityClass(), $definition, $rows, 'test', Context::createDefaultContext());
        static::assertCount(1, $structs);

        $first = $structs->first();
        $customFields = $first->get('customFields');

        static::assertIsArray($customFields);
        static::assertCount(2, $customFields);
        static::assertSame('Example', $customFields['custom_test_text']);
        static::assertNull($customFields['custom_test_check']);
    }

    public function testCustomFieldHydrationWithTranslationWithInheritance(): void
    {
        $definition = $this->registerDefinition(CustomFieldTestDefinition::class, CustomFieldTestTranslationDefinition::class);

        $hydrator = new EntityHydrator($this->getContainer());

        $id = Uuid::randomBytes();
        $context = $this->createContext();

        $rows = [
            [
                'test.id' => $id,
                'test.name' => 'example',
                'test.customTranslated' => '{"custom_test_text": null, "custom_test_check": "0"}',
                'test.translation.customTranslated' => '{"custom_test_text": null, "custom_test_check": "0"}',
                'test.translation.fallback_1.customTranslated' => '{"custom_test_text": null, "custom_test_check": null}',
                'test.parent.translation.customTranslated' => '{"custom_test_text": "PARENT DEUTSCH"}',
                'test.parent.translation.fallback_1.customTranslated' => '{"custom_test_text": "PARENT ENGLISH", "custom_test_check": "1"}',
            ],
        ];

        $structs = $hydrator->hydrate(new EntityCollection(), $definition->getEntityClass(), $definition, $rows, 'test', $context);
        static::assertCount(1, $structs);

        $first = $structs->first();
        $customFields = $first->get('customTranslated');
        static::assertSame('PARENT ENGLISH', $customFields['custom_test_text']);
        static::assertSame('1', $customFields['custom_test_check']);

        $rows = [
            [
                'test.id' => $id,
                'test.name' => 'example',
                'test.customTranslated' => '{"custom_test_text": null, "custom_test_check": null}',
                'test.translation.customTranslated' => '{"custom_test_text": null, "custom_test_check": null}',
                'test.translation.fallback_1.customTranslated' => '{"custom_test_text": null, "custom_test_check": null}',
                'test.parent.translation.customTranslated' => '{"custom_test_text": "PARENT DEUTSCH"}',
                'test.parent.translation.fallback_1.customTranslated' => '{"custom_test_text": "PARENT ENGLISH", "custom_test_check": "1"}',
            ],
        ];

        $structs = $hydrator->hydrate(new EntityCollection(), $definition->getEntityClass(), $definition, $rows, 'test', $context);
        $first = $structs->first();

        $customFields = $first->get('customTranslated');
        static::assertSame('PARENT ENGLISH', $customFields['custom_test_text']);
        static::assertSame('1', $customFields['custom_test_check']);

        $rows = [
            [
                'test.id' => $id,
                'test.name' => 'example',
                'test.customTranslated' => '{"custom_test_text": null, "custom_test_check": null}',
                'test.translation.customTranslated' => '{"custom_test_text": null, "custom_test_check": null}',
                'test.translation.fallback_1.customTranslated' => '{"custom_test_text": null, "custom_test_check": null}',
                'test.parent.translation.customTranslated' => '{"custom_test_text": null}',
                'test.parent.translation.fallback_1.customTranslated' => '{"custom_test_text": "PARENT ENGLISH", "custom_test_check": "0"}',
            ],
        ];

        $structs = $hydrator->hydrate(new EntityCollection(), $definition->getEntityClass(), $definition, $rows, 'test', $context);
        $first = $structs->first();

        $customFields = $first->get('customTranslated');
        static::assertSame('PARENT ENGLISH', $customFields['custom_test_text']);
        static::assertSame('0', $customFields['custom_test_check']);
    }

    public function testCustomFieldHydrationWithTranslationWithoutInheritance(): void
    {
        $definition = $this->registerDefinition(CustomFieldTestDefinition::class, CustomFieldTestTranslationDefinition::class);

        $hydrator = new EntityHydrator($this->getContainer());

        $id = Uuid::randomBytes();
        $context = $this->createContext(false);

        $rows = [
            [
                'test.id' => $id,
                'test.name' => 'example',
                'test.customTranslated' => '{"custom_test_text": null, "custom_test_check": "1"}',
                'test.translation.customTranslated' => '{"custom_test_text": null, "custom_test_check": "1"}',
                'test.translation.fallback_1.customTranslated' => '{"custom_test_text": "Example", "custom_test_check": null}',
            ],
        ];

        $structs = $hydrator->hydrate(new EntityCollection(), $definition->getEntityClass(), $definition, $rows, 'test', $context);
        static::assertCount(1, $structs);

        $first = $structs->first();
        $customFields = $first->get('customTranslated');
        static::assertSame('Example', $customFields['custom_test_text']);
        static::assertNull($customFields['custom_test_check']);
    }

    public function testCustomFieldHydrationWithoutTranslationWithInheritance(): void
    {
        $definition = $this->registerDefinition(CustomFieldTestDefinition::class, CustomFieldTestTranslationDefinition::class);

        $hydrator = new EntityHydrator($this->getContainer());

        $id = Uuid::randomBytes();
        $context = $this->createContext();

        $rows = [
            [
                'test.id' => $id,
                'test.name' => 'example',
                'test.custom' => '{"custom_test_text": null, "custom_test_check": null}',
                'test.custom.inherited' => '{"custom_test_text": "PARENT", "custom_test_check": "0"}',
            ],
        ];

        $structs = $hydrator->hydrate(new EntityCollection(), $definition->getEntityClass(), $definition, $rows, 'test', $context);
        static::assertCount(1, $structs);

        $first = $structs->first();
        $customFields = $first->get('custom');

        static::assertSame('PARENT', $customFields['custom_test_text']);
        static::assertSame('0', $customFields['custom_test_check']);

        $rows = [
            [
                'test.id' => $id,
                'test.name' => 'example',
                'test.custom' => '{"custom_test_text": null, "custom_test_check": "1"}',
            ],
        ];

        $structs = $hydrator->hydrate(new EntityCollection(), $definition->getEntityClass(), $definition, $rows, 'test', $context);
        static::assertCount(1, $structs);

        $first = $structs->first();
        $customFields = $first->get('custom');

        static::assertNull($customFields['custom_test_text']);
        static::assertSame('1', $customFields['custom_test_check']);
    }

    public function testSingleEntityDependencyWithDifferentlyLoadedAssociations(): void
    {
        $definition = $this->registerDefinition(
            SingleEntityDependencyTestRootDefinition::class,
            SingleEntityDependencyTestSubDefinition::class,
            SingleEntityDependencyTestDependencyDefinition::class,
            SingleEntityDependencyTestDependencySubDefinition::class,
        );

        $pickupPointId = Uuid::randomBytes();
        $warehouseId = Uuid::randomBytes();
        $zipcodeId = Uuid::randomBytes();
        $countryId = Uuid::randomBytes();

        $context = $this->createContext();

        $rowWithoutWarehouseZipcodeHydration = [
            'test.id' => $pickupPointId,
            'test.name' => 'PickupPoint',
            'test.warehouseId' => $warehouseId,
            'test.warehouse.id' => $warehouseId,
            'test.warehouse.name' => 'Warehouse',
            'test.warehouse.zipcodeId' => $zipcodeId,
            'test.zipcodeId' => $zipcodeId,
            'test.zipcode.id' => $zipcodeId,
            'test.zipcode.zipcode' => '00000',
            'test.zipcode.countryId' => $countryId,
            'test.zipcode.country.id' => $countryId,
            'test.zipcode.country.iso' => 'DE',
        ];

        $rowWithWarehouseZipcodeHydration = [
            'test.id' => $pickupPointId,
            'test.name' => 'PickupPoint',
            'test.warehouseId' => $warehouseId,
            'test.warehouse.id' => $warehouseId,
            'test.warehouse.name' => 'Warehouse',
            'test.warehouse.zipcodeId' => $zipcodeId,
            'test.warehouse.zipcode.id' => $zipcodeId,
            'test.warehouse.zipcode.zipcode' => '00000',
            'test.warehouse.zipcode.countryId' => $countryId,
            'test.zipcodeId' => $zipcodeId,
            'test.zipcode.id' => $zipcodeId,
            'test.zipcode.zipcode' => '00000',
            'test.zipcode.countryId' => $countryId,
            'test.zipcode.country.id' => $countryId,
            'test.zipcode.country.iso' => 'DE',
        ];

        $hydrator = new EntityHydrator($this->getContainer());
        $structsWithoutWarehouseZipcodeHydration = $hydrator->hydrate(new EntityCollection(), $definition->getEntityClass(), $definition, [$rowWithoutWarehouseZipcodeHydration], 'test', $context);
        static::assertNotNull($structsWithoutWarehouseZipcodeHydration->first()->get('zipcode')->get('country'));
        static::assertEquals(Uuid::fromBytesToHex($countryId), $structsWithoutWarehouseZipcodeHydration->first()->get('zipcode')->get('country')->get('id'));

        $hydrator = new EntityHydrator($this->getContainer());
        $structsWithWarehouseZipcodeHydration = $hydrator->hydrate(new EntityCollection(), $definition->getEntityClass(), $definition, [$rowWithWarehouseZipcodeHydration], 'test', $context);
        static::assertNotNull($structsWithWarehouseZipcodeHydration->first()->get('zipcode')->get('country'));
        static::assertEquals(Uuid::fromBytesToHex($countryId), $structsWithWarehouseZipcodeHydration->first()->get('zipcode')->get('country')->get('id'));
    }

    private function addLanguage(string $id, ?string $rootLanguage): void
    {
        $translationCodeId = Uuid::randomHex();
        $languageRepository = $this->getContainer()->get('language.repository');
        $languageRepository->create(
            [
                [
                    'id' => $id,
                    'parentId' => $rootLanguage,
                    'name' => $id,
                    'localeId' => $this->getLocaleIdOfSystemLanguage(),
                    'translationCode' => [
                        'id' => $translationCodeId,
                        'name' => 'x-' . $translationCodeId,
                        'code' => 'x-' . $translationCodeId,
                        'territory' => $translationCodeId,
                    ],
                ],
            ],
            Context::createDefaultContext()
        );
    }

    private function createContext(bool $inheritance = true): Context
    {
        $rootLanguageId = Uuid::randomHex();
        $this->addLanguage($rootLanguageId, null);

        return new Context(
            new SystemSource(),
            [],
            Defaults::CURRENCY,
            [$rootLanguageId, Defaults::LANGUAGE_SYSTEM],
            Defaults::LIVE_VERSION,
            1.0,
            $inheritance
        );
    }
}

class FkExtensionFieldTest extends EntityDefinition
{
    public function getEntityName(): string
    {
        return 'fk_extension_test';
    }

    protected function defineFields(): FieldCollection
    {
        return new FieldCollection([
            (new IdField('id', 'id'))->addFlags(new ApiAware(), new PrimaryKey()),
            (new StringField('name', 'name'))->addFlags(new ApiAware()),
            (new FkField('normal_fk', 'normalFk', ProductDefinition::class))->addFlags(new ApiAware()),

            (new FkField('extended_fk', 'extendedFk', ProductDefinition::class))->addFlags(new ApiAware(), new Extension()),
        ]);
    }
}
